const navIList = Components.interfaces.navIList;
const nsIWebProgressListener = Components.interfaces.nsIWebProgressListener;
    
var gUrlnav = null;
function urlnavLoad() 
{
  gUrlnav = new Urlnav();
  gUrlnav.init();
};

function urlnavUnload() 
{
  gUrlnav.destroy();
  gUrlnav = null;
};

function Urlnav() {};
Urlnav.prototype = {
  _els: null,
  _branch: null,
  _bundle: null,
  _slideshow: null,
  _sheet: null,
  
  init: function()
  {
    //add progress listener
    getBrowser().addProgressListener(this, Components.interfaces.nsIWebProgress.NOTIFY_ALL);     
    
    //get pref branch
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService); 
    this._branch = pbs.getBranch("urlnav.");
        
    //get string bundle
    this._bundle = document.getElementById("navBundle");
    
    //setup variables related to dom
    this._els = new Object();
    this._els["capture"] = document.getElementById("navCapture");
    this._els["release"] = document.getElementById("navRelease");
    this._els["up"] = document.getElementById("navUp");
    this._els["down"] = document.getElementById("navDown");
    this._els["start"] = document.getElementById("navStart");
    this._els["slideshow"] = document.getElementById("navSlideshow");

    //setup slideshow object
    this._slideshow = new Object();
    this._slideshow["enabled"] = false;
    this._slideshow["dir"] = null;
    this._slideshow["timer"] = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);

    //get the urlnav stylesheet
    for (var i in document.styleSheets) {
      if (document.styleSheets[i].href.match(
        /chrome.*\/skin.*\/urlnav.css$/ )) {
        this._sheet = document.styleSheets[i];
        break;
      };
    };
    
    //updat style rules
    this._updateRules();
    
    //set the caption
    var url = getBrowser().currentURI.spec;
    this._els["capture"].setAttribute("tooltiptext", this._bundle.getFormattedString("urlnav.capture.enabled", [url]));      

    //do first time prompt
    if (this._branch.getBoolPref("first"))
      window.setTimeout("gUrlnav.first()", 10);
          
    //add preference observer
    var pbi = this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver("", this, false);    
  },
  
  destroy: function()
  {
    //remove progress listener
    try {
      getBrowser().removeProgressListener(this);
    } catch(e) {};
    
    //remove preference observer
    try {
      var pbi = this._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
      pbi.removeObserver("", this);
      pbi = null;
    } catch(e) {};
    
    //stop timers
    this._slideshow["timer"].cancel();
    
    //destroy variables
    this._els = null;
    this._branch = null;
    this._bundle = null;
    this._slideshow = null;
    this._sheet = null;
  },
  
  first: function()
  {
    //setup prompt service
    var title = this._bundle.getString("urlnav.first.title");
    var message = this._bundle.getString("urlnav.first.message");
    var checkbox = this._bundle.getString("urlnav.first.checkbox");
    var showAgain = {value: false};
    
    //prompt for customize  
    const IPS = Components.interfaces.nsIPromptService;
    var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(IPS);
    var rv = ps.confirmEx(window, title, message, 
                       (IPS.BUTTON_TITLE_YES * IPS.BUTTON_POS_0) + 
                       (IPS.BUTTON_TITLE_NO * IPS.BUTTON_POS_1),
                       null, null, null, checkbox, showAgain);
    
    // customize if yes                    
    if (rv == 0)
      BrowserCustomizeToolbar(); 
    
    //set first time pref
    this._branch.setBoolPref("first", showAgain.value);  
  },
  
  capture: function(aURL)
  {
    //get list information
    var url = aURL;
    var referrer = null;
    if (!aURL) {
      var webNavigation = getWebNavigation();
      url = webNavigation.currentURI.spec;
      referrer = webNavigation.referringURI;
    };
    var list = Components.classes["@mozdev.org/urlnav/list;1"].createInstance(navIList);
    window.openDialog("chrome://urlnav/content/capture.xul", "_blank", "dependent=yes, modal=yes, dialog=no", url, list, referrer);
    if (list.start.url == "")
      return;
      
    //attach list to browser
    var browser = getBrowser().mCurrentBrowser;
    browser.list = list;
   
    //update ui
    this._update();
  },
  
  release: function()
  { 
    //remove list
    var browser = getBrowser().mCurrentBrowser;
    browser.list = null;
    
    //update ui
    this._update();  
  },
  
  move: function(aEvent, aDirection)
  {
    //navigate to location
    var list = getBrowser().mCurrentBrowser.list;
    list.move(aEvent, aDirection, window);  
     
    //update ui
    this._update();            
  },
    
  slideshow: function(aEvent)
  {
    //stop slideshow
    if (this._slideshow["enabled"]) {
      this._slideshow["enabled"] = false;
      this._slideshow["dir"] = null;
      this._slideshow["timer"].cancel();
      this._update();
      return;
    };
    
    //start slideshow
    this._slideshow["enabled"] = true;
    var direction = this._branch.getIntPref("slideshow.dir"); 
    if (!aEvent)
      this._slideshow["dir"] = direction;
    else {
      if (aEvent.ctrlKey)
        this._slideshow["dir"] = (direction == navIList.MOVE_DOWN) ? navIList.MOVE_UP : navIList.MOVE_DOWN;
      else
        this._slideshow["dir"] = direction;
    };
    
    //navigate to first page
    this.move(null, this._slideshow["dir"]);
  },
  
  onDragOver: function(aEvent, aFlavour, aDragSession)
  {
    aEvent.target.setAttribute("dragover", "true");
    return true; 
  },
  
  onDragExit: function (aEvent, aDragSession)
  {
    aEvent.target.removeAttribute("dragover");
  },
  
  onDrop: function (aEvent, aXferData, aDragSession)
  {
    var xferData = aXferData.data.split("\n");
    var url = xferData[0] ? xferData[0] : xferData[1];
    if (url)
      this.capture(null, url);
  },
  
  getSupportedFlavours: function ()
  {
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
    flavourSet.appendFlavour("text/x-moz-url");
    flavourSet.appendFlavour("text/unicode");
    return flavourSet;  
  },

  onLocationChange: function(aWebProgress, aRequest, aLocation) 
  {
    //update ui
    this._update();
      
    if (aWebProgress.DOMWindow != content)
      return;
    
    //update capture button
    try {
      var url = getBrowser().currentURI.spec;
      this._els["capture"].setAttribute("tooltiptext", this._bundle.getFormattedString("urlnav.capture.enabled", [url]));      
    } catch(e) {
      this._els["capture"].setAttribute("tooltiptext", this._bundle.getString("urlnav.capture.disabled"));      
    };
  },

  onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) 
  {  
    if (aWebProgress.DOMWindow != content)
      return;
      
    if (!aRequest)
      return;  
              
    if (aStateFlags & nsIWebProgressListener.STATE_STOP) {
      //start slideshow timer
      if (this._slideshow["enabled"])
        this._slideshow["timer"].initWithCallback(this, this._branch.getIntPref("slideshow.delay") * 1000,Components.interfaces.nsITimer.TYPE_ONE_SHOT);     
    };
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic != "nsPref:changed")
      return;
                
    //do updates based on pref changed
    switch (aData) {
      case "valid.enabled":  
      case "history.enabled":
        var browser = getBrowser().mCurrentBrowser;
        var show = (typeof(browser.list) == "object" && browser.list != null) ? true : false;      
        if (!show)
          return;
          
        var list = browser.list;
        list.init(list.start.url, list.builder, list.startIndex, list.endIndex, list.referrer);
        this._update();
        break;
      case "valid.color":
      case "history.color":
        this._updateRules();      
        break;    
      case "buttons.dir":
      case "buttons.hide":
        this._update();
        break;
    };    
  },
  
  notify: function(aTimer)
  {
    //do slideshow move
    if (this._slideshow["enabled"])
      this.move(null, this._slideshow["dir"]);  
  },
  
  //needed for progress listener  
  onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {},
  onLinkIconAvailable: function(aHref) {},      
  onProgressChange: function (aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {},
  onSecurityChange: function(aWebProgress, aRequest, aState) {},

  _update: function()
  {
    var browser = getBrowser().mCurrentBrowser;
    
    //set button direction
    var item = document.getElementById("urlnavItem");
    if (item)
      item.setAttribute("dir", this._branch.getCharPref("buttons.dir"));

    //set button display
    var show = (typeof(browser.list) == "object" && browser.list != null) ? true : false;
    if (show) {
      for (var i in this._els) {
        this._els[i].removeAttribute("disabled");
        this._els[i].removeAttribute("shown");
        this._els[i].removeAttribute("history");
        this._els[i].removeAttribute("valid");  
      };

      //set tooltips
      this._els["start"].setAttribute("tooltiptext", this._bundle.getFormattedString("urlnav.start.enabled", [browser.list.start.url]));    
      this._els["up"].setAttribute("tooltiptext", this._bundle.getFormattedString("urlnav.up.enabled", [browser.list.up.url]));
      this._els["down"].setAttribute("tooltiptext", this._bundle.getFormattedString("urlnav.down.enabled", [browser.list.down.url]));
      
      //set history
      if (this._branch.getBoolPref("history.enabled")) {
        this._els["start"].setAttribute("history", (browser.list.start.history)? "true" : "false");    
        this._els["up"].setAttribute("history", (browser.list.up.history)? "true" : "false");
        this._els["down"].setAttribute("history",  (browser.list.down.history)? "true" : "false");
      };
      
      //set valid
      if (this._branch.getBoolPref("valid.enabled")) {
        this._els["start"].setAttribute("valid", (browser.list.start.valid)? "true" : "false");    
        this._els["up"].setAttribute("valid", (browser.list.up.valid)? "true" : "false");
        this._els["down"].setAttribute("valid",  (browser.list.down.valid)? "true" : "false");
      };
            
    } else {
      for (var i in this._els) {
        if (i == "capture")
          continue;
        if (this._branch.getBoolPref("buttons.hide"))
          this._els[i].setAttribute("shown", "false"); 
        else
          this._els[i].removeAttribute("shown");         
        this._els[i].setAttribute("disabled", "true");
        this._els[i].removeAttribute("history");
        this._els[i].removeAttribute("valid");
      };
      
      //set tooltips
      this._els["start"].setAttribute("tooltiptext", this._bundle.getString("urlnav.start.disabled"));    
      this._els["up"].setAttribute("tooltiptext", this._bundle.getString("urlnav.up.disabled"));
      this._els["down"].setAttribute("tooltiptext", this._bundle.getString("urlnav.down.disabled"));
      
      //stop slideshow
      this._slideshow["enabled"] = false;
      this._slideshow["dir"] = null;
      this._slideshow["timer"].cancel();         
    };
    
    //do slideshow updates
    if (this._slideshow["enabled"]) {
      this._els["slideshow"].setAttribute("checked", "true");
      this._els["slideshow"].setAttribute("tooltiptext", this._bundle.getString("urlnav.slideshow.enabled"));
    } else {
      this._els["slideshow"].removeAttribute("checked");
      this._els["slideshow"].setAttribute("tooltiptext", this._bundle.getString("urlnav.slideshow.disabled"));
    };    
  },
  
  _updateRules: function()
  {
    if (!this._sheet)
      return;
      
    //delete the old rules
    for (var x=0; x<this._sheet.cssRules.length ; ++x) {
      if (this._sheet.cssRules[x].cssText.indexOf("history=") == 0) {
        this._sheet.deleteRule(x);
        --x;
      };
      if (this._sheet.cssRules[x].cssText.indexOf("valid=") == 0) {
        this._sheet.deleteRule(x);
        --x;
      };
    };
    
    //insert new history rule
    var color = this._branch.getCharPref("history.color");
    var rule = ".urlnav-button[history=\"true\"] { -moz-appearance: none; background-color: " + color + "; }";
    this._sheet.insertRule(rule, this._sheet.cssRules.length);
    
    //insert new failed rule
    color = this._branch.getCharPref("valid.color");
    rule = ".urlnav-button[valid=\"false\"] { -moz-appearance: none; background-color: " + color + "; }";
    this._sheet.insertRule(rule, this._sheet.cssRules.length);
  }
};